// @flow
import _ from 'lodash';
import {compose, defaultProps, mapProps, withHandlers, withProps} from 'recompose';
import {connect} from 'react-redux';
import {default as rest} from '../rest';

export default (hocParams: Object) => compose(
	withProps(props => ({
		resolvedEntity: _.isFunction(hocParams.entity) ? hocParams.entity(props) : hocParams.entity,
		resolvedKey   : _.isFunction(hocParams.key) ? hocParams.key(props) : hocParams.key,
	})),

	defaultProps({faFetched: {}}),
	withProps(props => ({keyStatePath: `${hocParams.statePath || 'fetchApi'}.${props.resolvedKey}`})),
	withProps(props => ({keyStatePath: `${hocParams.statePath || 'fetchApi'}.${props.resolvedKey}`})),

	connect(
		(state, props: {
			faFetched: Object,
			keyStatePath: string,
			resolvedKey: string,
		}) => ({
			faFetched: {
				...props.faFetched,

				[props.resolvedKey]: {
					..._.pick(_.get(state, props.keyStatePath), ['data', 'lastResponse', 'loading']),
					baseURL: hocParams.baseURL || 'http://local.api.app.bluavenu.com:8000',
				},
			},
		}),

		dispatch => ({dispatch}),
	),

	mapProps(props => ({
		...props,

		faFetched: {
			...props.faFetched,

			[props.resolvedKey]: {
				...props.faFetched[props.resolvedKey],

				// WARN Track changes in queryParams/urlParams and resync. Common procedure for all components composed with base.js
				// WARN Update docs with example of details view with queryParams populated with id
				// Let connect() to be called and fill up props before queryParams() called being passed with props
				queryParams: _.isFunction(hocParams.queryParams) ? hocParams.queryParams(props) : {},

				reset: (mutation: string) => props.dispatch(
					rest({
						baseURL: props.faFetched[props.resolvedKey].baseURL,
						entity : props.resolvedEntity,
						key    : props.resolvedKey,
					}).actions[props.resolvedKey].reset(mutation)
				),

				// There is no way to put .sync into .faFetched within connect()
				sync: (urlParams?: {}, params?: {}, callback?: () => void) => props.dispatch(
					rest({
						baseURL: props.faFetched[props.resolvedKey].baseURL,
						entity : props.resolvedEntity,
						key    : props.resolvedKey,
					}).actions[props.resolvedKey].sync(urlParams, params, callback)
				),

				urlParams: _.isFunction(hocParams.urlParams) ? hocParams.urlParams(props) : {},
			},
		},
	})),

	withHandlers({
		getQueryParams: props => _.get(
			props,
			'getQueryParams',
			propsParam => propsParam.faFetched[props.resolvedKey].queryParams
		),

		// WARN Probably _.isEqual will cause performance issues, go for immutability!
		hasPropsChanged: () => (params: {
			nextProps: Object,
			props: Object,
			trackProps: Array<string>,
		}) => _.some(
			params.trackProps,

			prop => !_.isEqual(
				params.props.faFetched[params.props.resolvedKey][prop],
				params.nextProps.faFetched[params.nextProps.resolvedKey][prop]
			),
		),
	}),
);
